/*
 * Copyright (c) 2012, Felix Opatz
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 */

#define _CRT_SECURE_NO_WARNINGS

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <windows.h>

#define BKSIZE 1024

void print_help(void)
{
   printf("copyclip -- copies data to or from the clipboard\n");
   printf("\n");
   printf("usage:\n");
   printf("   copyclip [option]\n");
   printf("\n");
   printf("options:\n");
   printf("   -h   print this help message\n");
   printf("   -p   paste data from clipboard\n");
}

int to_clipboard(void)
{
   int failed = 0;
   int c;
   char *buf = NULL;
   unsigned int len = 0;
   unsigned int pos = 0;

   while (!failed && (c = getchar()) != EOF) {
      if (pos + 1 > len) {
         len += BKSIZE;
         buf = realloc(buf, len);
      }
      if (buf) {
         buf[pos] = c;
         pos++;
      } else {
         fprintf(stderr, "error allocating memory: %s\n",
            strerror(errno));
         failed = 1;
      }
   }

   if (!failed && buf) {
      if (OpenClipboard(NULL)) {
         HGLOBAL hMem;
         char *pMem;

         EmptyClipboard();
         hMem = GlobalAlloc(GMEM_MOVEABLE, pos + 1);
         if (hMem) {
            pMem = GlobalLock(hMem);
            memcpy(pMem, buf, pos);
            pMem[pos] = '\0';
            GlobalUnlock(hMem);
            if (SetClipboardData(CF_OEMTEXT, hMem) != NULL) {
               printf("pasted %lu characters to the clipboard\n", pos);
            } else {
               fprintf(stderr, "error setting clipboard data: %lu\n",
                  GetLastError());
               failed = 1;
            }
         } else {
            fprintf(stderr, "error allocating memory: %lu\n", GetLastError());
            failed = 1;
            CloseClipboard();
         }
      } else {
         fprintf(stderr, "error opening clipboard: %lu\n", GetLastError());
         failed = 1;
      }
   }

   return failed ? EXIT_FAILURE : EXIT_SUCCESS;
}

int from_clipboard(void)
{
   int failed = 0;

   if (IsClipboardFormatAvailable(CF_OEMTEXT)) {
      if (OpenClipboard(NULL)) {
         HANDLE hData;
         const char *pData;

         hData = GetClipboardData(CF_OEMTEXT);
         if (hData) {
            pData = GlobalLock(hData);
            if (pData) {
               printf("%s", pData);
               GlobalUnlock(hData);
            } else {
               fprintf(stderr, "error locking data: %lu\n",
                  GetLastError());
               failed = 1;
            }
         } else {
            fprintf(stderr, "error getting clipboard data: %lu\n",
               GetLastError());
            failed = 1;
         }
      } else {
         fprintf(stderr, "error opening clipboard: %lu\n", GetLastError());
         failed = 1;
      }
   } else {
      /* No clipboard data available -- no message */
      failed = 1;
   }

   return failed ? EXIT_FAILURE : EXIT_SUCCESS;
}

int main(int argc, char *argv[])
{
   int n;
   int result = EXIT_SUCCESS;
   int paste = 0;
   int help = 0;

   for (n = 1; n < argc; n++) {
      if (!strcmp(argv[n], "-h"))
         help = 1;
      else if (!strcmp(argv[n], "-?"))
         help = 1;
      else if (!strcmp(argv[n], "-p"))
         paste = 1;
   }

   if (help) {
      print_help();
   } else if (paste) {
      result = from_clipboard();
   } else {
      result = to_clipboard();
   }

   return result;
}
